<?php

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

namespace TYPO3\CMS\Install\CoreVersion;

class MajorRelease
{
    protected $version;
    protected $lts;
    protected $title;
    protected $maintenanceWindow;

    public function __construct($version, $title, MaintenanceWindow $maintenanceWindow, $lts = null)
    {
        $this->version = $version;
        $this->title = $title;
        $this->maintenanceWindow = $maintenanceWindow;
        $this->lts = $lts;
    }

    public static function fromApiResponse(array $response)
    {
        $maintenanceWindow = MaintenanceWindow::fromApiResponse($response);
        $ltsVersion = isset($response['lts']) ? (string)$response['lts'] : null;

        return new self((string)$response['version'], $response['title'], $maintenanceWindow, $ltsVersion);
    }

    public function getVersion()
    {
        return $this->version;
    }

    public function getLts()
    {
        return $this->lts;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getMaintenanceWindow()
    {
        return $this->maintenanceWindow;
    }
}
